# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'puppet_x/enterprisemodules/db2/core'

Puppet::Type.newtype(:db2_database) do
  include EasyType
  include PuppetX::EnterpriseModules::Db2::Type
  include PuppetX::EnterpriseModules::Db2::Access

  desc <<-DESC
  This resource allows you to manage an DB2 Database. Here is an example on how you can do this:

      db2_database {'db2inst1/MYDB':
        ensure => 'present',
        rec_his_retentn      => 90,
        section_actuals      => 'NONE',
        self_tuning_mem      => 'ON',
        seqdetect            => 'YES',
      }

  This statement will ensure a database `MYDB` on instance `db2inst1`.
  DESC

  to_get_raw_resources do
    databases = []
    db2_instance_names.each do |instance|
      databases += databases_details_on(instance).collect { |e| e.merge(:instance => instance) }
    end
    databases
  end

  ensurable

  on_create do
    self.class.databases_on_cache_reset(instance_name)
    db2 "create database #{database_name}", :instance => instance_name
    # rubocop: disable Style/HashSyntax
    apply_updates(force: true)
    # rubocop: enable Style/HashSyntax
    nil
  end

  on_modify do
    apply_updates
    nil
  end

  on_destroy do
    self.class.databases_on_cache_reset(instance_name)
    db2 "drop database #{database_name}", :instance => instance_name
    nil
  end

  def apply_updates(force: false)
    #
    # Because we want to manage the exceptions of the update command, we construct the command,
    # instead of letting easy_type construct the command
    #
    properties = provider.send(:property_values, 'create')
    return nil if properties.empty?

    begin
      db2 "update database config for #{database_name} using  #{properties.join(' ')}", { :instance => instance_name, :database => database_name }
    rescue Puppet::ExecutionFailure => e
      case e.message
      when /DB20000I  The UPDATE DATABASE CONFIGURATION/
        # Nothing
      when /SQL1363W  One or more of the parameters submitted for immediate modification \nwere not changed dynamically/
        if allow_restart.to_s == 'true' || force
          Puppet.debug 'Restart after some static parameters where changed'
          restart_manager
        else
          Puppet.warning 'Auto restart disabled, but some of the requested changes are static and need a database restart.'
        end
      else
        raise
      end
    end
  end

  def stop_manager
    Puppet.debug Puppet::Util::Execution.execute('. ~/sqllib/db2profile; db2stop', :uid => instance_name)
  end

  def start_manager
    Puppet.debug Puppet::Util::Execution.execute('. ~/sqllib/db2profile; db2start', :uid => instance_name)
  end

  def restart_manager
    stop_manager
    start_manager
  end

  map_titles_to_attributes([%r{((.*)/(.*))}, [:name, :instance_name, :database_name]])

  parameter :name
  parameter :instance_name
  parameter :database_name
  parameter :allow_restart

  property :act_sortmem_limit
  property :alt_collate
  property :app_ctl_heap_sz
  property :appgroup_mem_sz
  property :appl_memory
  property :applheapsz
  property :archretrydelay
  property :authn_cache_duration
  property :authn_cache_users
  property :auto_del_rec_obj
  property :auto_maint
  property :auto_reval
  property :autorestart
  property :avg_appls
  property :backup_pending
  property :blk_log_dsk_ful
  property :blocknonlogged
  property :cf_catchup_trgt
  property :cf_db_mem_sz
  property :cf_gbp_sz
  property :cf_lock_sz
  property :cf_sca_sz
  property :catalogcache_sz
  property :chngpgs_thresh
  property :codepage
  property :codeset
  property :collate_info
  property :connect_proc
  property :ctrl_file_recov_path
  property :cur_commit
  property :database_consistent
  property :database_level
  property :database_memory
  property :date_compat
  property :dbheap
  property :db_mem_thresh
  property :ddl_compression_def
  property :ddl_constraint_def
  property :dec_arithmetic
  property :dec_to_char_fmt
  property :decflt_rounding
  property :dft_degree
  property :dft_extent_sz
  property :dft_loadrec_ses
  property :dft_mttb_types
  property :dft_prefetch_sz
  property :dft_queryopt
  property :dft_refresh_age
  property :dft_schemas_dcc
  property :dft_sqlmathwarn
  property :dft_table_org
  property :discover_db
  property :dlchktime
  property :enable_xmlchar
  property :encrlib
  property :encropts
  property :extbl_location
  property :extbl_strict_io
  property :extended_row_sz
  property :failarchpath
  property :groupheap_ratio
  property :hadr_db_role
  property :hadr_local_host
  property :hadr_local_svc
  property :hadr_peer_window
  property :hadr_remote_host
  property :hadr_remote_inst
  property :hadr_remote_svc
  property :hadr_replay_delay
  property :hadr_spool_limit
  property :hadr_ssl_host_val
  property :hadr_ssl_label
  property :hadr_syncmode
  property :hadr_target_list
  property :hadr_timeout
  property :indexrec
  property :large_aggregation
  property :locklist
  property :locktimeout
  property :log_appl_info
  property :log_ddl_stmts
  property :log_disk_cap
  property :log_retain_status
  property :logarchcompr1
  property :logarchcompr2
  property :logarchmeth1
  property :logarchmeth2
  property :logarchopt1
  property :logarchopt2
  property :logbufsz
  property :logfilsiz
  property :loghead
  property :logindexbuild
  property :logpath
  property :logprimary
  property :logsecond
  property :max_log
  property :maxappls
  property :maxfilop
  property :maxlocks
  property :min_dec_div_3
  property :mincommit
  property :mirrorlogpath
  property :mon_act_metrics
  property :mon_deadlock
  property :mon_locktimeout
  property :mon_lockwait
  property :mon_lw_thresh
  property :mon_lck_msg_lvl
  property :mon_obj_metrics
  property :mon_pkglist_sz
  property :mon_req_metrics
  property :mon_rtn_data
  property :mon_rtn_execlist
  property :mon_uow_data
  property :mon_uow_execlist
  property :mon_uow_pkglist
  property :multipage_alloc
  property :nchar_mapping
  property :newlogpath
  property :num_db_backups
  property :num_freqvalues
  property :num_iocleaners
  property :num_ioservers
  property :num_log_span
  property :num_quantiles
  property :numarchretry
  property :numsegs
  property :number_compat
  property :opt_direct_wrkld
  property :overflowlogpath
  property :page_age_trgt_gcr
  property :page_age_trgt_mcr
  property :pagesize
  property :pckcachesz
  property :pl_stack_trace
  property :priv_mem_thresh
  property :rec_his_retentn
  property :restore_pending
  property :restrict_access
  property :rollfwd_pending
  property :section_actuals
  property :self_tuning_mem
  property :seqdetect
  property :sheapthres_shr
  property :smtp_server
  property :softmax
  property :sortheap
  property :sql_ccflags
  property :stat_heap_sz
  property :stmt_conc
  property :stmtheap
  property :string_units
  property :suspend_io
  property :systime_period_adj
  property :territory
  property :trackmod
  property :tsm_mgmtclass
  property :tsm_nodename
  property :tsm_owner
  property :tsm_password
  property :user_exit_status
  property :util_heap_sz
  property :varchar2_compat
  property :vendoropt
  property :wlm_admission_ctrl
  property :wlm_agent_load_trgt
  property :wlm_collect_int
  property :wlm_cpu_limit
  property :wlm_cpu_shares
  property :wlm_cpu_share_mode
  #
  # DB2 V11 new properties
  #
  property :auto_cg_stats
  property :auto_db_backup
  property :auto_reorg
  property :auto_runstats
  property :auto_sampling
  property :auto_stats_views
  property :auto_stmt_stats
  property :auto_tbl_maint
  property :repl_site_id
  #
  # HADR properties
  #
  property :hadr_role
end
