# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'puppet_x/enterprisemodules/db2/core'

Puppet::Type.newtype(:db2_exec) do
  include EasyType
  include PuppetX::EnterpriseModules::Db2::Access

  desc <<-DESC
    This type allows you run a specific SQL statement or an sql file on a specified instance and database.

        db2_exec { "db2inst1/MYDB/drop table application_users":
          username => 'app_user',
          password => 'password,'
        }

    This statement will execute the sql statement `drop table application_users` on the databse 'MYDB' on instance 'db2inst1'.

    You can use the `unless` parameter to only execute the statement in certain states. If the query specified in the
    `unless` parameter returns one or more records, the main statement is skipped.

        db2_exec{ "create synonym ${user}.${synonym} for USER.${synonym}":
          unless  => "select * from all_synonyms where owner=\'${user}\' and synonym_name=\'${synonym}\'",
        }

    You can also execute a script.

        db2_exec{"db2inst1/MYDB/@/tmp/do_some_stuff.sql":
          username  => 'app_user',
          password  => 'password,'
          logoutput => on_failure,  # can be true, false or on_failure
        }

    This statement will run the sqlscript `/tmp/do_some_stuff.sql` on the database named `MYDB` on instance `db2inst1`. Use the `unless`
    parameter to just execute the script in certain situations.

    When you don't specify the username and the password, the type will connect as `sysdba`.


  DESC

  map_titles_to_attributes([
                             %r{^((.*)/(.*)/(.*))$}, [:name, :instance_name, :database_name, :statement]
                           ])

  def refresh
    return unless provider.need_to_execute?

    provider.execute
    if parameters[:statement].sensitive
      Puppet.notice "#{path}: defined 'statement' as '[redacted]'"
    else
      Puppet.notice "#{path}: defined 'statement' as '#{statement}'"
    end
  end

  parameter :name
  property  :statement
  parameter :instance_name
  parameter :database_name

  parameter :timeout
  parameter :cwd
  parameter :logoutput
  parameter :password
  parameter :username
  parameter :unless
  parameter :onlyif
  parameter :refreshonly
  parameter :report_errors
  parameter :mark_as_error

  # db2_autorequire(:db2_user, :username)

  def onlyif_specified?
    !self[:onlyif].nil?
  end

  def unless_specified?
    !self[:unless].nil?
  end

  def report_errors?
    case self[:report_errors].to_s
    when 'true'
      true
    when 'false'
      false
    else
      fail 'invalid value for report_errors'
    end
  end
end
