# frozen_string_literal: true

#
#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'puppet_x/enterprisemodules/db2/core'

Puppet::Type.newtype(:db2_instance) do
  include EasyType
  include PuppetX::EnterpriseModules::Db2::Type
  include PuppetX::EnterpriseModules::Db2::Access

  desc <<-DESC
  This resource allows you to manage an DB2 instance. Here is an example on how to use this:

      db2_instance { 'db2inst2':
        ensure              => 'present',
        agent_stack_sz      => 1024,
        agentpri            => 'SYSTEM',
        alternate_auth_enc  => 'AES_ONLY',
        aslheapsz           => 15,
        audit_buf_sz        => 1000,
        authentication      => 'SERVER_ENCRYPT',
        catalog_noauth      => 'NO',
      }

  #### Manage the OS user or not

  If the `manage_user` parameter is set to true, Puppet will create the OS users required for an instance. If you need more control
  over the creation process, you can set the parameter to false and manage the user(s) yourself. Look at the parameters:

  - instance_password
  - instance_uid
  - instance_group
  - instance_home
  - fence_user
  - fence_password
  - fence_uid
  - fence_group
  - fence_home

  for more details.

  When removing an instance, the parameter will determine if Puppet removes the OS users.

  DESC

  to_get_raw_resources do
    db2_instances
  end

  ensurable

  on_create do
    if manage_users.to_s == 'true'
      create_instance_user
      create_fence_user
    end
    create_instance
  end

  on_modify do
    [:db2, 'update database manager configuration using', { :instance => name, :skip_without_properties => true }]
  end

  on_destroy do
    stop_manager if provider.state == 'running'
    delete_instance
    if manage_users.to_s == 'true'
      delete_instance_user
      delete_fence_user
    end
  end

  after_modify do
    if restart.to_s == 'true'
      stop_manager
      start_manager
    end
  end

  parameter :name
  parameter :instance_password
  parameter :instance_uid
  parameter :instance_group
  parameter :instance_home
  parameter :fence_user
  parameter :fence_password
  parameter :fence_uid
  parameter :fence_group
  parameter :fence_home
  parameter :instance_type
  parameter :authentication_type
  parameter :port
  parameter :manage_users
  parameter :location
  parameter :restart

  # property :db2_version
  property :state

  property :cpuspeed
  property :comm_bandwidth
  property :numdb
  property :federated
  property :tp_mon_name
  property :dft_account_str
  property :jdk_path
  property :diaglevel
  property :notifylevel
  property :diagpath
  property :alt_diagpath
  property :datalinks
  property :diagsize
  property :dft_mon_bufpool
  property :dft_mon_lock
  property :dft_mon_sort
  property :dft_mon_stmt
  property :dft_mon_table
  property :dft_mon_timestamp
  property :dft_mon_uow
  property :health_mon
  property :sysadm_group
  property :sysctrl_group
  property :sysmaint_group
  property :sysmon_group
  property :clnt_pw_plugin
  property :clnt_krb_plugin
  property :group_plugin
  property :local_gssplugin
  property :srv_plugin_mode
  property :srvcon_gssplugin_list
  property :srvcon_pw_plugin
  property :srvcon_auth
  property :authentication
  property :alternate_auth_enc
  property :catalog_noauth
  property :trust_allclnts
  property :trust_clntauth
  property :fed_noauth
  property :dftdbpath
  property :mon_heap_sz
  property :java_heap_sz
  property :audit_buf_sz
  property :instance_memory
  property :agent_stack_sz
  property :sheapthres
  property :dir_cache
  property :aslheapsz
  property :rqrioblk
  property :util_impact_lim
  property :agentpri
  property :num_poolagents
  property :num_initagents
  property :max_coordagents
  property :max_connections
  property :keepfenced
  property :fenced_pool
  property :num_initfenced
  property :indexrec
  property :tm_database
  property :resync_interval
  property :spm_name
  property :spm_log_file_sz
  property :spm_max_resync
  property :spm_log_path
  property :svcename
  property :discover
  property :discover_inst
  property :ssl_svr_keydb
  property :ssl_svr_stash
  property :ssl_svr_label
  property :ssl_svcename
  property :ssl_cipherspecs
  property :ssl_versions
  property :ssl_clnt_keydb
  property :ssl_clnt_stash
  property :max_querydegree
  property :intra_parallel
  property :federated_async
  property :fcm_num_buffers
  property :fcm_buffer_size
  property :fcm_num_channels
  property :fcm_parallelism
  property :conn_elapse
  property :max_connretries
  property :max_time_diff
  property :start_stop_time
  property :wlm_dispatcher
  property :wlm_disp_concur
  property :wlm_disp_cpu_shares
  property :wlm_disp_min_util
  property :comm_exit_list
  property :cur_eff_arch_lvl
  property :cur_eff_code_lvl
  property :keystore_type
  property :keystore_location
  property :python_path

  def fence_username
    return fence_user if fence_user

    name.gsub('inst', 'fenc')
  end

  def create_instance_user
    command = '/usr/sbin/useradd -m -c "created by Puppet db2_config" '
    command += "-u #{instance_uid} " if instance_uid
    command += "-g #{instance_group} " if instance_group
    command += "-p #{instance_password} " if instance_password
    command += "-d #{instance_home} " if instance_home
    command += name
    Puppet::Util::Execution.execute(command)
  end

  def create_fence_user
    command = '/usr/sbin/useradd -m -c "created by Puppet db2_config" '
    command += "-u #{fence_uid} " if fence_uid
    command += "-g #{fence_group} " if fence_group
    command += "-p #{fence_password} " if fence_password
    command += "-d #{fence_home} " if fence_home
    command += fence_username
    Puppet::Util::Execution.execute(command)
  end

  def create_instance
    command = "#{location}/instance/db2icrt -u #{fence_username} "
    command += "-s #{instance_type} " if instance_type
    command += "-a #{authentication_type} " if authentication_type
    command += "-p #{port} " if port
    command += name
    Puppet.debug Puppet::Util::Execution.execute(command)
    self.class.db2_instance_cache_reset
  end

  def delete_instance
    command = "#{location}/instance/db2idrop #{name}"
    Puppet.debug Puppet::Util::Execution.execute(command)
    self.class.db2_instance_cache_reset
  end

  def delete_fence_user
    delete_user(name)
  end

  def delete_instance_user
    delete_user(fence_username)
  end

  def delete_user(user)
    Puppet::Util::Execution.execute("/usr/sbin/userdel -rf #{user}")
  end

  def stop_manager
    Puppet.debug Puppet::Util::Execution.execute('. ~/sqllib/db2profile; db2stop', :uid => name)
  end

  def start_manager
    Puppet.debug Puppet::Util::Execution.execute('. ~/sqllib/db2profile; db2start', :uid => name)
  end
end
