# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'puppet_x/enterprisemodules/db2/core'

Puppet::Type.newtype(:db2_object_grant) do
  include EasyType
  include PuppetX::EnterpriseModules::Db2::Type
  include PuppetX::EnterpriseModules::Db2::Access

  validate do
    raise 'object name should use "OWNER.OBJECT"  syntax' unless object_name.split('.').size == 2
  end

  desc <<-DESC
  This puppet type allows you to manage grants/permissions on database objects. Here is an example:

      db2_object_grant { 'db2inst1/MYDB/USER/MYUSER->MYSCHEMA.MYTABLE':
        permissions            => ['CONTROL'],
        provider               => 'db2',
        with_grant_permissions => ['ALTER', 'DELETE', 'INDEX', 'INSERT', 'REFERENCE', 'SELECT', 'UPDATE'],
      }

  This code tell's Puppet that on database `MYDB` on instance `dbs2inst1`, the user `MYUSER` has `CONTROL` permission
  on the table `MYTABLE` in schema `MYSCHEMA`. It can also grant the rights ALTER', 'DELETE', 'INDEX', 'INSERT', 'REFERENCE', 'SELECT', 'UPDATE
  to other users.

  DESC

  def self.merge_value(current_grant, grant, p)
    current_grant[p] = 'Y' if grant[p] == 'Y'
    current_grant[p] = 'G' if grant[p] == 'G'
  end

  def self.merge_permissions(existing_grants, grant)
    instance    = grant.column_data(:instance)
    database    = grant.column_data(:database)
    granteetype = grant.column_data('GRANTEETYPE')
    grantee     = grant.column_data('GRANTEE').upcase
    schema      = grant.column_data('TABSCHEMA').upcase
    table_name  = grant.column_data('TABNAME').upcase
    current_grant = existing_grants.find do |a_grant|
      next false unless instance    == a_grant.column_data(:instance)
      next false unless database    == a_grant.column_data(:database)
      next false unless granteetype == a_grant.column_data('GRANTEETYPE')
      next false unless grantee     == a_grant.column_data('GRANTEE').upcase
      next false unless schema      == a_grant.column_data('TABSCHEMA').upcase
      next false unless table_name  == a_grant.column_data('TABNAME').upcase

      true
    end

    %w[CONTROLAUTH ALTERAUTH DELETEAUTH INDEXAUTH INSERTAUTH REFAUTH SELECTAUTH SELECTAUTH].each { |p| merge_value(current_grant, grant, p) }
  end

  def self.join_duplicates(grants)
    processed = []
    processed_grants = []
    grants.each do |grant_record|
      instance    = grant_record.column_data(:instance)
      database    = grant_record.column_data(:database)
      granteetype = grant_record.column_data('GRANTEETYPE')
      grantee     = grant_record.column_data('GRANTEE').upcase
      schema      = grant_record.column_data('TABSCHEMA').upcase
      table_name  = grant_record.column_data('TABNAME').upcase
      title = "#{instance}/#{database}/#{granteetype}/#{grantee}->#{schema}.#{table_name}"
      if processed.include?(title)
        merge_permissions(processed_grants, grant_record)
      else
        processed_grants << grant_record
        processed << title
      end
    end
    processed_grants
  end

  to_get_raw_resources do
    grants = []
    db2_instance_names.each do |instance|
      primary_database_names_on(instance).each do |database|
        grants += grants_on(instance, database).collect { |e| e.merge(:instance => instance, :database => database) }
      end
    end
    join_duplicates(grants)
  end

  on_create do
    nil # Let the permissions property do its work
  end

  on_modify do
    nil # Let the permissions property do its work
  end

  on_destroy do
    fail "we shouldn't be in on_destroy in db2_object_grant"
  end

  map_titles_to_attributes([%r{((.*)/(.*)/(.+)/(.+)->(.+))}, [:name, :instance_name, :database_name, :grantee_type, :grantee, :object_name]])

  parameter :name
  parameter :instance_name
  parameter :database_name
  parameter :object_name
  parameter :grantee_type
  parameter :grantee

  property  :permissions
  property  :with_grant_permissions
end
