# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:permissions, :array_matching => :all) do
  include EasyType
  include PuppetX::EnterpriseModules::Db2::Access

  desc <<-DESC
    The permissions on the specified object, granted to the specified user.

    This is a required array of rights to grant on the user object combination

        db2_object_grant{ ...:
          permissions => ['execute', 'select'],
        }

    When this list contains less rights then currently granted, the the extra rights will
    be revoked from the user. If the permissions list contains more rights then currently
    granted in the database, the extra rights will be granted to the user.

    If you want to make sure no rights are granted, you must use an empty array.

        db2_object_grant{ ...:
          permissions => ['execute', 'select'],
        }

  DESC

  isrequired
  data_type('Array[String[1]]')

  to_translate_to_resource do |raw_resource|
    permissions = []
    permissions << 'CONTROL' if raw_resource.column_data('CONTROLAUTH') == 'Y'
    permissions << 'ALTER' if raw_resource.column_data('ALTERAUTH') == 'Y'
    permissions << 'DELETE' if raw_resource.column_data('DELETEAUTH') == 'Y'
    permissions << 'INDEX' if raw_resource.column_data('INDEXAUTH') == 'Y'
    permissions << 'INSERT' if raw_resource.column_data('INSERTAUTH') == 'Y'
    permissions << 'REFERENCE' if raw_resource.column_data('REFAUTH') == 'Y'
    permissions << 'SELECT' if raw_resource.column_data('SELECTAUTH') == 'Y'
    permissions << 'UPDATE' if raw_resource.column_data('UPDATEAUTH') == 'Y'
    permissions
  end

  def insync?(is)
    is = [] if is.to_s == 'absent' || is.nil?
    is.sort == munged_should.sort
  end

  def change_to_s(_from, _to)
    return_value = []
    return_value << "revoked the #{revoked_rights.join(',')} right(s)" unless revoked_rights.empty?
    return_value << "granted the #{granted_rights.join(',')} right(s)" unless granted_rights.empty?
    return_value.join(' and ')
  end

  on_apply do
    db2(grant(granted_rights), :instance => resource.instance_name, :database => resource.database_name) if granted_rights.any?
    db2(revoke(revoked_rights), :instance => resource.instance_name, :database => resource.database_name) if revoked_rights.any?
    nil
  end

  private

  def munged_should
    should.to_s == 'absent' || should.nil? ? [] : should
  end

  def revoke(rights)
    if resource.grantee == 'PUBLIC'
      "revoke #{rights.join(',')} on #{resource.object_name} from #{resource.grantee}"
    else
      "revoke #{rights.join(',')} on #{resource.object_name} from #{resource.grantee_type} #{resource.grantee}"
    end
  end

  def grant(rights)
    if resource.grantee == 'PUBLIC'
      "grant #{rights.join(',')} on #{resource.object_name} to #{resource.grantee}"
    else
      "grant #{rights.join(',')} on #{resource.object_name} to #{resource.grantee_type} #{resource.grantee}"
    end
  end

  def current_permissions
    return [] if provider.permissions == :absent

    provider.permissions
  end

  def revoked_rights(rights = current_permissions)
    rights - munged_should
  end

  def granted_rights(rights = current_permissions)
    munged_should - rights
  end
end
