# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'puppet_x/enterprisemodules/db2/core'

Puppet::Type.newtype(:db2_validation) do
  include EasyType
  include PuppetX::EnterpriseModules::Db2::Access

  desc <<-DESC
  This type allows you to validate the contents of your DB2 database. Here is an example on how you can
  use this:

      db2_validation { "db2inst1/MYDB/No user data in system tablespaces":
        check                   => "select tabschema,tabname,tbspace from syscat.tables where tabschema not in ('ADMINISTRATOR','SYSIBM','SYSTOOLS') and tbspace in ('SYSCATSPACE','SYSTOOLSPACE','SYSTOOLSTMPSPACE','TEMPSPACE')",
        expected_number_of_rows => 0,
        fail_message            => 'User table %<tabschema>s.%<tabname>s not allowed system tablespace %<tbspace>s',
        report_as               => 'command',
        fail_command            => "/bin/send_to_montoring --node ${::fqdn} %<output>s",
      }

  The Puppet statement executes a query on the database and report a warning when more the 0 records are
  found. It uses the returned records of the query to generate a good reading error or warning message.
  DESC

  map_titles_to_attributes([
                             %r{^((.*)/(.*)/(.*))$}, [:name, :instance_name, :database_name, :check]
                           ])

  to_get_raw_resources do
    []
  end

  on_modify do
    report
  end

  def report
    message = "Validation #{name}  failed with messages : \n#{@messages.join("\n")}"
    case report_as
    when /warning|WARNING/
      Puppet.warning message
    when /error|ERROR/
      raise Puppet::Error, message
    when /command|COMMAND/
      # Issue a waning and execute the fail_command
      Puppet.warning message
      final_command = format(fail_command, :messages => @messages.join("\n"))
      Puppet.debug Puppet::Util::Execution.execute(
        final_command,
        :failonfail => true,
        :combine => true,
        :override_locale => false
      )
    else
      raise "#{report_as} is not a valid value"
    end
  end

  def refresh
    records = execute_check

    report if records.count != expected_number_of_rows
  end

  def execute_check
    records = execute_query(check)
    records = records.collect do |r|
      r = r.transform_keys(&:downcase).transform_keys(&:to_sym)
      r.transform_values(&:strip)
    end
    @messages = records.collect { |r| format(fail_message, r) }
  end

  def execute_query(query)
    options = { :instance => instance_name, :database => database_name, :parse => true }
    options[:username] = username if username.to_s != 'absent'
    options[:password] = password if password.to_s != 'absent'
    sql query, options
  end

  parameter :name
  property  :check
  parameter :instance_name
  parameter :database_name
  parameter :refreshonly

  parameter :timeout
  parameter :password
  parameter :username
  parameter :report_as
  parameter :expected_number_of_rows
  parameter :fail_message
  parameter :fail_command
end
