# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:granted, :array_matching => :all) do
  include EasyType
  include PuppetX::EnterpriseModules::Db2::GrantProperty

  desc <<-DESC
  The grants you want to make sure are granted to the `db2_user`, `db2_group``, or `db2_role`. It is different from the
  `grants` property in the sense that this is not a full list of the rights, but just the rights
  that are granted to the user

  Here is an example with an `db2_user`:

      db2_user {'db2inst1/MYDB/my_user':
        ...
        granted => ['CREATETAB'],
        ...
      }

  Here is an example with an `db2_role`:

      db2_role {'db2inst1/MYDB/my_role':
        ...
        granted => ['CREATETAB'],
        ...
      }

      **WARNING**

      When you us both the `grants` and `granted` property, Puppet will give you a warning about conflicting values. In the end it **WILL** ensure
      that the rights specified for the `revoked` property are removed. This way ensuring the most secure configuration.


  DESC

  to_translate_to_resource do |_raw_resource|
    nil
  end

  #
  # because the order may differ, but they are still the same,
  # to decide if they are equal, first do a sort on is and should
  #
  def insync?(is)
    is = [] if is.to_s == 'absent' || is.nil?
    is.sort == assembled_value.sort
  end

  def revoked_rights
    current_rights - value
  end

  #
  # Assemble the actua value based on all grant and revoked properties
  #
  def assembled_value
    granted = resource[:granted].nil? ? [] : resource[:granted]
    revoked = resource[:revoked].nil? ? [] : resource[:revoked]
    (value + granted - revoked).uniq
  end

  def granted_rights
    assembled_value - current_rights
  end

  data_type('Array[String[1]]')
end
