# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:revoked, :array_matching => :all) do
  include EasyType
  include PuppetX::EnterpriseModules::Db2::GrantProperty

  validate do |value|
    fail 'You need to specify an Array instead of a comma separated string' if /,/.match?(value.to_s)
  end

  desc <<-DESC
    The grants you want to make sure are revoked from the `db2_user`, `db2_group` or `db2_role`. It is different from the
    `grants` property in the sense that this is not a full list of the rights, but just the rights
    that are **NOT** granted to the user

    Here is an example when using `db2_user`:

        db2_user {'db2inst1/MYDB/my_user':
          ...
          revoked => ['CREATETAB'],
        }

    Here is an example when using `db2_role`:

        db2_role {'db2inst1/MYDB/my_role':
          ...
          revoked => ['CREATETAB'],
        }

      **WARNING**

      When you us both the `grants` and `revoked` property, Puppet will give you a warning about conflicting values. In the end it **WILL** ensure
      that the rights specified for the `revoked` property are removed. This way ensuring the most secure configuration.
    #{'  '}

  DESC

  to_translate_to_resource do |_raw_resource|
    nil
  end

  data_type('Optional[Array[String[1]]]')

  def insync?(_is)
    should.none? { |r| current_rights.include?(r) }
  end

  def revoked_rights
    current_rights & value
  end
end
