#
# db2_config::fact_caching
#
# @summary This class ensure's that the db2_config facts that can be cached and are cached with the correct settings.
# You can change all TTL settings for the facts if an other setting is more fitting for your setup.
#
# @param [Boolean] enabled
#    When set to true, this enables facter caching for the db2_config module facts.
#    Beware that you need to have Facter version 4.1.0 or higher to enabled this. When you set this value to true, automatically TTL settings for most of the db2_config facts are set to a default value. For most installations, the default settings are fine.
#    When you set this value to `true`, the [facter setting `force-dot-resolution`](https://tickets.puppetlabs.com/browse/PUP-11089) is enabled. You have to ensure that all of your facts keep on working with this setting.
#
# @param [Optional[String[1]]] db2_databases_ttl
#    The Time to Live for the `db2_databases` fact.
#
# @param [Optional[String[1]]] db2_instances_ttl
#    The Time to Live for the `db2_instances` fact.
#
# @param [Optional[String[1]]] db2_schemas_ttl
#    The Time to Live for the `db2_schemas` fact.
#
# @param [Optional[String[1]]] db2_tablespaces_ttl
#    The Time to Live for the `db2_tablespaces` fact.
#
# @param [Optional[String[1]]] db2_version_ttl
#    The Time to Live for the `db2_version` fact.
#
#
# See the file "LICENSE" for the full license governing this code.
#
class db2_config::fact_caching (
  Optional[String[1]] $db2_databases_ttl,
  Optional[String[1]] $db2_instances_ttl,
  Optional[String[1]] $db2_schemas_ttl,
  Optional[String[1]] $db2_tablespaces_ttl,
  Optional[String[1]] $db2_version_ttl,
  Boolean             $enabled
) {
  if $enabled {
    include db2_config::clear_caches
    #
    # If enabled, we need to enable the force-dot-resolution
    # setting for facter.
    #
    unless defined(Fact_setting['force-dot-resolution']) {
      fact_setting { 'force-dot-resolution':
        value => true,
      }
    }

    fact_config { 'db2_databases':
      ttl => $db2_databases_ttl,
    }

    fact_config { 'db2_instances':
      ttl => $db2_instances_ttl,
    }

    fact_config { 'db2_schemas':
      ttl => $db2_schemas_ttl,
    }

    fact_config { 'db2_tablespaces':
      ttl => $db2_tablespaces_ttl,
    }

    fact_config { 'db2_version':
      ttl => $db2_version_ttl,
    }
  } else {
    #
    # On puppet versions older than V6, the required libs
    # for fact_config are not installed. Because it also doesn't
    # serve any purpose, we skip this part on those versions.
    #
    if versioncmp($clientversion, '6.0.0') != -1 {
      #
      # Disable fact caching for all db2_config facts
      #
      $db2_config_facts = [
        'db2_databases',
        'db2_instances',
        'db2_schemas',
        'db2_tablespaces',
        'db2_version',
      ]

      fact_config { $db2_config_facts:
        ttl => absent,
      }
    }
} }
